#!/usr/bin/env python
"""
This module is the GUI for Twisted Life.
"""


#Import Modules
import os, pygame, random
from pygame.locals import *
import twisted_life

if not pygame.font: print 'Warning, fonts disabled'
if not pygame.mixer: print 'Warning, sound disabled'

#game constants
NUM_LIVES         = 4
HEIGHT = WIDTH    = 468
SCREENRECT        = Rect(0, 0, HEIGHT, WIDTH)
UNIVERSE_FRACTION = 0.95
MAX_ROUND_SCORE   = 1000
SCORE_REDUCER     = 0.99

# globals
score                   = 0
fractionOfMaxRoundScore = 1.0

def display(universe, rects, screen, playerCoords, all, background):
    for i in range(universe.diameter()):
        for j in range(universe.diameter()):
            rect = rects[j][i]
            if universe.isAlive(j, i):
                color = Color('black')
            else:
                color = Color('white')
            pygame.draw.rect(screen, color, rect)
    centerRect = rects[universe.radius][universe.radius]
    if universe.isAlive(universe.radius, universe.radius):
        color = Color('red')
    else:
        color = Color('green')
    pygame.draw.rect(screen, color, centerRect)

    updateTextSprites(all, screen, background)

    playerRect = rects[playerCoords[0]][playerCoords[1]]
    if universe.isAlive(playerCoords[0], playerCoords[1]):
        color = Color('red')
    else:
        color = Color('blue')
    pygame.draw.rect(screen, color, playerRect)
    #Draw Everything
    pygame.display.flip()

def roundLoop(clock, playerCoords, universe, rects, screen, all, background):
    global fractionOfMaxRoundScore

    fractionOfMaxRoundScore = 1.0
    
    playerX = playerCoords[0]
    playerY = playerCoords[1]
    #Main Loop
    while 1:
        clock.tick(60)

    #Handle Input Events
        moved = False
        for event in pygame.event.get():
            if event.type == QUIT:
                return 'quit'
            elif (event.type == KEYDOWN):
                if (event.key == K_ESCAPE):
                    return 'quit'
                elif (event.key == K_UP):
                    playerX -= 1
                    moved = True
                elif (event.key == K_DOWN):
                    playerX += 1
                    moved = True
                elif (event.key == K_LEFT):
                    playerY -= 1
                    moved = True
                elif (event.key == K_RIGHT):
                    playerY += 1
                    moved = True

        if not moved:                    
            pressedKeys = pygame.key.get_pressed()
            if (pressedKeys[K_UP]):
                playerX -= 1
            elif (pressedKeys[K_DOWN]):
                playerX += 1
            elif (pressedKeys[K_LEFT]):
                playerY -= 1
            elif (pressedKeys[K_RIGHT]):
                playerY += 1

        playerX = (playerX + universe.diameter()) % universe.diameter()
        playerY = (playerY + universe.diameter()) % universe.diameter()
        playerCoords = (playerX, playerY)
        display(universe, rects, screen, playerCoords, all, background)
        if universe.isAlive(playerX, playerY):
            return 'lost'
        elif playerCoords == universe.getCenter():
            return 'won'

        universe.nextGen()
        display(universe, rects, screen, playerCoords, all, background)
        if universe.isAlive(playerX, playerY):
            return 'lost'
        elif playerCoords == universe.getCenter():
            return 'won'
        fractionOfMaxRoundScore *= SCORE_REDUCER

def initializePlayer(universe):
    onLiveSquare = True
    universeDiameter = universe.diameter()
    while onLiveSquare:
        playerX = random.randint(0, universeDiameter - 1)
        playerY = random.randint(0, universeDiameter - 1)
        onLiveSquare = universe.isAlive(playerX, playerY)

    return [playerX, playerY]
    
class Score(pygame.sprite.Sprite):
    def __init__(self):
        pygame.sprite.Sprite.__init__(self)
        self.font = pygame.font.Font(None, 20)
        self.font.set_italic(1)
        self.color = Color('white')
        self.lastscore = -1
        self.update()
        self.rect = self.image.get_rect().move(10, 460)

    def update(self):
        self.lastscore = score
        msg = "Score: %d, Max Round Score: %d" % (score, int(round(fractionOfMaxRoundScore * MAX_ROUND_SCORE)))
        self.image = self.font.render(msg, 0, self.color)

def updateTextSprites(all, screen, background):
    # clear/erase the last drawn sprites
    all.clear(screen, background)

    #update all the sprites
    all.update()

    #draw the scene
    dirty = all.draw(screen)
    pygame.display.update(dirty)

def main():
    """this function is called when the program starts.
       it initializes everything it needs, then runs in
       a loop until the function returns."""
    global score
#Initialize Everything
    pygame.init()
    screen = pygame.display.set_mode((WIDTH, HEIGHT + 15))
    background = pygame.Surface(SCREENRECT.size)
    pygame.display.set_caption('JEFF K!!!Z TWIZTED LIEF')
    pygame.mouse.set_visible(0)

    # Initialize Game Groups
    all = pygame.sprite.RenderUpdates()

    #assign default groups to each sprite class
    Score.containers = all

    #initialize our starting sprites
    if pygame.font:
        all.add(Score())

    universeRadius = 32
    universeDiameter = 2 * universeRadius + 1
    
    universe = twisted_life.TwistedLife(universeRadius)
    universe.nextGen()
    cellWidth = WIDTH / universe.diameter()
    cellHeight = HEIGHT / universe.diameter()

    rects = []
    for i in range(universe.diameter()):
        row = []
        y = i * cellHeight
        for j in range(universe.diameter()):
            x = j * cellWidth
            rect = pygame.Rect(x, y, cellWidth, cellHeight)
            row.append(rect)
        rects.append(row)

#Display The Background
    pygame.display.flip()
    
#Prepare Game Objects
    clock = pygame.time.Clock()
    lossCount = 0
    roundNum = 1

    updateTextSprites(all, screen, background)

    while lossCount < NUM_LIVES:
        print 'Round ' + str(roundNum) + ': '
        playerCoords = initializePlayer(universe)
        display(universe, rects, screen, playerCoords, all, background)

        playerReady = False
        pygame.event.clear()
        while not playerReady:
            clock.tick(60)

        # Wait for player to start round.
            
            for event in pygame.event.get():
                if event.type == QUIT:
                    return
                elif (event.type == KEYDOWN):
                    if (event.key == K_ESCAPE):
                        return
                    elif (event.key == K_UP):
                        playerCoords[0] = (playerCoords[0] - 1 + universe.diameter()) % universe.diameter()
                        playerReady = True
                        
                        break
                    elif (event.key == K_DOWN):
                        playerCoords[0] = (playerCoords[0] + 1 + universe.diameter()) % universe.diameter()
                        playerReady = True

                        break
                    elif (event.key == K_LEFT):
                        playerCoords[1] = (playerCoords[1] - 1 + universe.diameter()) % universe.diameter()
                        playerReady = True

                        break
                    elif (event.key == K_RIGHT):
                        playerCoords[1] = (playerCoords[1] + 1 + universe.diameter()) % universe.diameter()
                        playerReady = True

                        break
                    
        result = roundLoop(clock, playerCoords, universe, rects, screen, all, background)
        if result == 'lost':
            print '\tYou ran into a live cell.  You lose the round.'
            lossCount += 1
        elif result == 'won':
            print '\tYou reached the goal.  You win the round.'
            score += int(round(fractionOfMaxRoundScore * MAX_ROUND_SCORE))
        elif result == 'quit':
            return

        print "Your score:", score

        updateTextSprites(all, screen, background)

        pygame.time.wait(2000)
        universe = twisted_life.TwistedLife(universeRadius)
        universe.nextGen()
        roundNum += 1
    
#Game Over
    print "Your final score:", score
    pygame.time.wait(5000)

#this calls the 'main' function when this script is executed
if __name__ == '__main__': main()
